C ----------------------------------------------------------------------
C Francais
C     Programme CADM: 
C     Test de congruence entre plusieurs matrices de distance, utilisant 
C     la statistique W de Kendall.
C     
C     Principe du test:
C
C     1. Lire les matrices de distance (fichiers ASCII) et les ecrire dans 
C        les lignes d'un tableau sous forme depliee.
C     2. Transformer les distances de chaque matrice en rangs.
C     3. Calculer le coefficient de concordance W de Kendall.
C     4. Permuter chaque matrice a la facon du test de Mantel et calculer W*.
C     5. Repeter l'etape 4 un grand nombre de fois.
C     6. Ajouter W a la distribution des W*, puis calculer la 
C        probabilite permutationnelle associee a W.
C
C     A l'interieur d'un test particulier, les trois statistiques W, "chisq" et 
C     S sont monotones l'une a l'autre. Puisqu'elles produisent la meme 
C     probabilite permutationnelle, elles sont donc equivalentes pour les tests 
C     par permutation.
C
C     Note: W est en relation lineaire avec la moyenne des r de Spearman entre
C           toutes les paires de "juges" (matrices) (Siegel 1956: 232).
C
C     Le programme permet de traiter des matrices de distance asymetriques.
C     Ce traitement n'est utilise que si l'une des matrices demeure asymetrique,
C     i.e., n'est pas transformee en matrice symetrique dans TransMat.
C
C                           ----------------
C
C     Lecture des matrices de distance (fichiers ASCII):
C
C     Il peut y avoir un seul fichier de donnees contenant toutes les matrices
C     de distance, ou encore un fichier distinct pour chaque matrice.
C     Chaque matrice de distance est en ASCII, carree et avec diagonale.
C
C     nmax = nombre maximum d'objets (n = nombre reel d'objets)
C     matmax = n. max. de mat. de distance (m = n. reel de mat. de distance)
C     kmax = longueur max. de deux demi-matrices depliees (k = longueur reelle)
C
C     'DMax' contient toutes les matrices de distance, l'une sous l'autre.
C
C     'Table' contient (lignes) les matrices depliees. Dans le cas de matrices
C     asymetriques, la triangulaire superieure est ecrite d'abord, suivie de
C     la triangulaire inferieure.
C
C     'R' = vecteur contenant la somme des rangs de chaque colonne de 'Table'.
C     'data' est un vecteur de travail contenant une ligne de 'Table'.
C     'rang' est un vecteur de travail utilise dans TRI avec 'data'.
C
C                                Pierre Legendre, novembre 2000 - septembre 2001
C
C ------------------------------------------------------------------------------
C English
C     Program CADM: This program computes a test of the congruence 
C     among several distance matrices, using Kendall's W statistic.
C
C     Principle of the test:
C
C     1. Read in the distance matrices (ASCII files) and write them out to a
C        table, each matrix being unfolded onto a row of that table.
C     2. Transform the distances from each matrix into ranks.
C     3. Compute Kendall's coefficient of concordance, W.
C     4. Permute each matrix in the Mantel way ("matrix permutation")
C        and compute W*.
C     5. Repeat step 4 a large number of times.
C     6. Add the reference value of W to the distribution of W* values.
C        Calculate the permutational probability associated to W.
C
C     Within a given permutation test, the three statistics W, "chisq", and 
C     S, are monotonic to one another. Since they produce the same  
C     permutational probability, they are equivalent for permutation testing.
C
C     Note: W is linearly related to the mean of the Spearman r statistics
C           among all pairs of "judges" (matrices) (Siegel 1956: 232).
C
C     The program allows the calculation of the test using asymmetric matrices.
C     This is used only if one of the matrices remains asymmetric, i.e.,
C     is not transformed into a symmetric matrix in TransMat.
C
C                           ----------------
C
C     Input distance matrices:
C
C     There may be a single data file containing all distance matrices, or a 
C     different file for each distance matrix.
C     Each distance matrix is in ASCII, square and with the diagonal.
C
C     nmax = maximum number of objects (n = actual number of objects)
C     matmax = max. no. distance matrices (m = actual no. distance matrices)
C     kmax = maximum length of two unfolded half-matrices (k = real length)
C
C     'DMax' contains all distance matrices, one after the other.
C
C     'Table' contains (in rows) the unfolded matrices. For asymmetric 
C     matrices, the upper triangular portion is written first, followed by
C     the lower triangular.
C
C     'R' is a vector contianing the sum of ranks of each column of 'Table'.
C     'data' is a work vector containing one row from 'Table'.
C     'rang' is a work vector used in TRI together with 'data'.
C
C                                Pierre Legendre, November 2000 - September 2001
C
C------------------------------------------------------------------------------
C
C Reference:
C
C Legendre, P., C. Levasseur and F.-J. Lapointe. A new test for assessing 
C    the congruence among distance matrices prior to phylogenetic analysis.  
C    Systematic Biology (submitted).
C
C------------------------------------------------------------------------------
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      Parameter (nmax=120,matmax=20,kmax=nmax*(nmax-1))
      Integer iordre(nmax),liste(kmax),igetS(matmax,matmax)
      Real*8 DMat(nmax*matmax,nmax),Table(matmax,kmax),T,R(kmax),
     + w(matmax),data(kmax),rang(kmax),rMantel(matmax,matmax),
     + probS(matmax,matmax)
C
C BBEdit is the 'creator' for output files
C      call F_Creator('R*ch')
C Langue du dialogue / Dialog language: 1 = francais; 2 = English
    2 write(*,*) 'Francais: tapez (1)'
      write(*,*) 'English:  type  (2)'
      read(*,*) langue
      if((langue.lt.1).or.(langue.gt.2)) goto 2
      open(7,file='CADM.out',status='NEW')
      if(langue.eq.1) then
         write(*,106)
         write(7,106)
         else
         write(*,206)
         write(7,206)
         endif
C Lecture des donnees, etc.
      call ReadData(n,m,DMat,Table,w,nmax,matmax,kmax,isymm,langue)
      write(*,*)
C      write(*,*)'Informations additionnelles (debug)? (0) non, (1) oui'
C      read(*,*) idebug
C      write(*,*)
      idebug=0
C
    4 if(langue.eq.1) then
         write(*,*) '(1) Calculer et tester la congruence seulement'
         write(*,*) 
     +      '(2) Calculer et tester Mantel sur les rangs seulement'
         write(*,*) '(3) Congruence et Mantel'
         else
         write(*,*) '(1) Compute and test congruence only'
         write(*,*) 
     +      '(2) Compute and test Mantel on ranked distances only'
         write(*,*) '(3) Both congruence and Mantel'
         endif
      read(*,*) ichoix
      if((ichoix.lt.1).or.(ichoix.gt.3)) goto 4
      write(*,*)
C
      if((ichoix.eq.1).or.(ichoix.eq.3)) then
      if(langue.eq.1) then
         write(*,*)'Combien de permutations pour le ',
     +'test de congruence?'
         write(*,*)'(E.g., 999, 9999, ...). Pas de test: tapez 0.'
         endif
      if(langue.eq.2) then
         write(*,*)'How many permutations for the test of congruence?'
         write(*,*)'(E.g. 999, 9999, ...). No permutation test: ',
     +'type 0.'
         endif
      read(*,*) npermK
      write(*,*)
      if(langue.eq.1) then
         if(isymm.eq.1) then
            write(7,108) npermK
            else
            write(7,109) npermK
            endif
         else
         if(isymm.eq.1) then
            write(7,208) npermK
            else
            write(7,209) npermK
            endif
         endif
      endif
C
      if(ichoix.ge.2) then
      if(langue.eq.1) then
         write(*,*)'Combien de permutations pour les tests de Mantel?'
         write(*,*)'(E.g., 999, 9999, ...). Pas de test: tapez 0.'
         endif
      if(langue.eq.2) then
         write(*,*)'How many permutations for the Mantel tests?'
         write(*,*)'(E.g. 999, 9999, ...). No permutation test: ',
     +'type 0.'
         endif
      read(*,*) npermS
      endif
C Demarrer l'horloge
      debut=secnds(0.0)
      call Prelim(n,m,DMat,Table,nmax,matmax,k,kmax,
     +        T,liste,data,rang,isymm,iseed,idebug)
      if((ichoix.eq.1).or.(ichoix.eq.3))
     +call Concord(n,m,DMat,Table,T,R,w,nmax,matmax,k,kmax,npermK,
     +        iordre,isymm,iseed,langue,idebug)
      if(ichoix.ge.2) 
     +call Mantel(n,m,nmax,matmax,k,kmax,DMat,igetS,probS,rMantel,
     +        iordre,npermS,isymm,iseed,langue,idebug)
C Duree du calcul
      write(*,*)
      fin=secnds(debut)
      if(langue.eq.1) then
         write(*,110) fin
         write(7,*)
         write(7,110) fin
         write(*,*) 
     +   "Les resultats se trouvent dans le fichier 'CADM.out'"
         write(*,*) 'Fin du programme.'
         else
         write(*,210) fin
         write(7,*)
         write(7,210) fin
         write(*,*) "Results are found in file 'CADM.out'"
         write(*,*) 'End of the program.'
         endif
      close(7)
      stop
  106 format(/' Test de la congruence'/
     +       ' de plusieurs matrices de distance (CMD)'//
     +       ' Pierre Legendre'/
     +       ' Departement de sciences biologiques'/
     +       ' Universite de Montreal.'/
     +       '             (c) Pierre Legendre, 2001'/)
  206 format(/' Test of congruence'/
     +       ' among several distance matrices (CADM)'//
     +       ' Pierre Legendre'/
     +       ' Departement de sciences biologiques'/
     +       ' Universite de Montreal.'/
     +       '             (c) Pierre Legendre, 2001'/)
  107 format(' * = parametre significatif au seuil 0.05')
  207 format(' * = parameter significant at 0.05 level')
  108 format(' Nombre de permutations =',i5,', matrices asymetriques'/)
  208 format(' Number of permutations =',i5,', non-symmetric matrices'/)
  109 format(' Nombre de permutations =',i5,', matrices symetriques'/)
  209 format(' Number of permutations =',i5,', symmetric matrices'/)
  110 format(' Duree du calcul:',f10.2,' sec.'/)
  210 format(' Computation time:',f10.2,' sec.'/)
      end
            
      Subroutine Concord(n,m,DMat,Table,T,R,w,nmax,matmax,k,kmax,
     +        npermK,iordre,isymm,iseed,langue,idebug)
      Integer iordre(nmax)
      Real*8 DMat(nmax*matmax,nmax),Table(matmax,kmax),R(kmax),S,
     +       Sper,T,w(matmax),chisq,chisqper,WKen,WKper,probper
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
C
C  Calcul et test de signification du coefficient de concordance W --
C
C  1. A tour de role, copier une ligne de 'Table' dans le vecteur 'data'.
C  2. Trier 'data' et le transformer en rangs.
C  3. Transcrire les distances transformees en rangs dans 'Table'.
C  4. Transcrire les distances transformees en rangs dans 'DMat'.
C  *** Les etapes 1 a 4 sont dans le sous-programme "Prelim"
C  5. Calculer le khi-carre et le coefficient de concordance (W) de Kendall.
C  6. Permuter toutes les matrices de distance et les recrire dans 'Table'.
C     Les objets sont les elements permutables: permutations a la Mantel.
C  7. Calculer W* sous permutation.
C  => Repeter les etapes 6 et 7 un grand nombre de fois.
C  8. Calculer la probabilite permutationnelle associee a W.
C
C Etape 5. Calculer le khi-carre et le coefficient de concordance (W) de Kendall
      iva=1
      Call WKendall(m,k,Table,T,R,w,matmax,kmax,S,chisq,WKen,iva,
     +idebug)
      if(langue.eq.1) write(7,101) chisq,WKen
      if(langue.eq.2) write(7,201) chisq,WKen
      write(7,100)
C Preparation en vue des permutations
      do 12 i=1,n
   12 iordre(i)=i
C
C Test global: on permute toutes les matrices, sauf la derniere * * * * * * * *
      iget=1
C Debut de la boucle de permutation - - - - - - - - - - - - - - - - - - - - - -
      iva=2
      do 30 iper=1,npermK
C
C Etape 6. Permuter les matrices (sauf la derniere) et les recrire dans 'Table'.
C Attention: pour les matrices asymetriques, les permutations se font entre les
C valeurs a l'interieur de chacune des portions triangulaires, independamment.
      do 20 ii=1,m-1
      ipos=(ii-1)*n
      call Permute(iseed,n,nmax,iordre)
C      if(idebug.eq.1) then
C         write(*,301) (iordre(i), j=1,n)
C         do 16 i=1,m*n
C   16    write(*,302) (DMat(i,j), j=1,n)
C         endif
C  301 format(20i4)
C  302 format(20f6.3)
      jj=0
      if(isymm.eq.1) then
         do 14 i=1,(n-1)
         do 14 j=(i+1),n
         jj=jj+1
         ia=iordre(i)
         ib=iordre(j)
         if(ia.lt.ib) then
            Table(ii,jj)=DMat(ipos+ia,ib)
            else
            Table(ii,jj)=DMat(ipos+ib,ia)
            endif
   14    continue
         do 16 i=2,n
         do 16 j=1,(i-1)
         jj=jj+1
         ia=iordre(i)
         ib=iordre(j)
         if(ia.gt.ib) then
            Table(ii,jj)=DMat(ipos+ia,ib)
            else
            Table(ii,jj)=DMat(ipos+ib,ia)
            endif
   16    continue
        else
         do 18 i=1,(n-1)
         do 18 j=(i+1),n
         jj=jj+1
   18    Table(ii,jj)=DMat(ipos+iordre(i),iordre(j))
        endif
   20 continue
C
C Etape 7. Calculer W sous permutation.
      Call WKendall(m,k,Table,T,R,w,matmax,kmax,Sper,chisqper,WKper,
     +   iva,idebug)
      if(Sper.ge.S) iget=iget+1
C
   30 continue
C Fin de la boucle de permutation - - - - - - - - - - - - - - - - - - - - - - -
C
C Etape 8. Calculer la probabilite permutationnelle associee a W.
      probper=dfloat(iget)/dfloat(npermK+1)
      if(langue.eq.1) then
         write(7,102)
         write(7,103) npermK,probper
         else
         write(7,202)
         write(7,203) npermK,probper
         endif
      write(7,100)
C
C Test particulier pour chaque matrice  * * * * * * * * * * * * * * * * * * * *
      do 50 ii=1,m
      ipos=(ii-1)*n
      iget=1
C     Recopier d'abord toutes les matrices (non permutees) dans 'Table'.
      do 38 iii=1,m
      ipos2=(iii-1)*n
      jj=0
      if(isymm.eq.1) then
         do 32 i=1,(n-1)
         do 32 j=(i+1),n
         jj=jj+1
   32    Table(iii,jj)=DMat(ipos2+i,j)
         do 34 i=2,n
         do 34 j=1,(i-1)
         jj=jj+1
   34    Table(iii,jj)=DMat(ipos2+i,j)
        else
         do 36 i=1,(n-1)
         do 36 j=(i+1),n
         jj=jj+1
   36    Table(iii,jj)=DMat(ipos2+i,j)
        endif
   38 continue
C Debut de la boucle de permutation - - - - - - - - - - - - - - - - - - - - - -
      iva=2
      do 46 iper=1,npermK
C
C Etape 6. Permuter la matrice ii et la recrire dans 'Table'.
      call Permute(iseed,n,nmax,iordre)
      jj=0
      if(isymm.eq.1) then
         do 40 i=1,(n-1)
         do 40 j=(i+1),n
         jj=jj+1
         ia=iordre(i)
         ib=iordre(j)
         if(ia.lt.ib) then
            Table(ii,jj)=DMat(ipos+ia,ib)
            else
            Table(ii,jj)=DMat(ipos+ib,ia)
            endif
   40    continue
         do 42 i=1,(n-1)
         do 42 j=(i+1),n
         jj=jj+1
         ia=iordre(i)
         ib=iordre(j)
         if(ia.gt.ib) then
            Table(ii,jj)=DMat(ipos+ia,ib)
            else
            Table(ii,jj)=DMat(ipos+ib,ia)
            endif
   42    continue
        else
         do 44 i=1,(n-1)
         do 44 j=(i+1),n
         jj=jj+1
   44    Table(ii,jj)=DMat(ipos+iordre(i),iordre(j))
        endif
C
C Etape 7. Calculer W sous permutation.
      Call WKendall(m,k,Table,T,R,w,matmax,kmax,Sper,chisqper,WKper,
     +   iva,idebug)
      if(Sper.ge.S) iget=iget+1
C
   46 continue
C Fin de la boucle de permutation - - - - - - - - - - - - - - - - - - - - - - -
C
C Etape 8. Calculer la probabilite permutationnelle associee a W.
      probper=dfloat(iget)/dfloat(npermK+1)
      if(langue.eq.1) then
         write(7,105) ii,ii
         write(7,104) npermK,probper
         else
         write(7,205) ii,ii
         write(7,204) npermK,probper
         endif
      write(7,200)
   50 continue 
C Fin des tests particuliers  * * * * * * * * * * * * * * * * * * * * * * * * *
      return
  100 format(' ---------------------------------------'/)
  200 format(' -------------'/)
  101 format(' Khi-carre de Friedman = ',f15.5/
     +       ' W de Kendall          = ',f15.5/
     +       ' avec correction pour valeurs liees'/)
  201 format(' Friedman''s Chi-square = ',f15.5/
     +       ' Kendall''s W           = ',f15.5/
     +       ' with correction for tied ranks'/)
  102 format(' Test global. H0: Incongruence de toutes les matrices.'/
     +   ' Toutes les matrices sont permutees, de facon ',
     +'independante.'/)
  202 format(' Global test. H0: Incongruence of all matrices.'/
     +       ' All matrices are independently permuted.'/)
  103 format(' Prob(khi-carre,',i6,' perm.) =',f8.5/)
  203 format(' Prob(chi-square,',i6,' perm.) =',f8.5/)
  104 format(' Prob(khi-carre,',i6,' perm.) =',f8.5/)
  204 format(' Prob(chi-square,',i6,' perm.) =',f8.5/)
  105 format(
     +' Test de la matrice',i3,'. H0: Cette matrice est incongruente.'/
     +       ' La matrice',i3,' est permutee.'/)
  205 format(' Test for matrix',i3,'. H0: This matrix is incongruent.'/
     +       ' Matrix',i3,' is permuted.'/)
      end
      
      Subroutine Mantel(n,m,nmax,matmax,k,kmax,DMat,igetS,probS,
     +rMantel,iordre,npermS,isymm,iseed,langue,idebug)
      Integer iordre(nmax),igetS(matmax,matmax)
      Real*8 DMat(nmax*matmax,nmax),rMantel(matmax,matmax),
     +       probS(matmax,matmax)
      Real*8 sx,sy,sx2,sy2,sxy,r,dflnpSp1
C Calcul d'une matrice de correlations de Mantel basees sur les rangs dans DMat.
C
C Pour des donnees liees, Zar (1999) recommande d'utiliser la formule generale
C du coefficient de correlation sur les rangs, corrigee pour les donnees liees,
C plutot que la formule condensee du r de Spearman (Numerical Ecology, p. 197).
C On pourrait aussi utiliser la formule 5.2.
      if(langue.eq.1) write(7,101)
      if(langue.eq.2) write(7,201)
      dflnk=dfloat(k)
C Preparation en vue des permutations
      do 10 i=1,n
   10 iordre(i)=i
C
C      if(idebug.eq.1) then
C         do 5 k=1,m
C         write(*,*) 'k =',k
C         ipos=(k-1)*n
C         write(*,*) 'ipos =',ipos
C         do 2 i=1,n
C         write(*,110) (DMat(ipos+i,j), j=1,n)
C    2    continue
C         write(*,*)
C    5    continue
C         endif
C
C We use the property that the sum of the first n integers is n*(n+1)/2
      sx=0.5*dflnk*(dflnk+1.0)
      sy=sx
C
      do 20 ki=1,m-1
      do 20 kj=(ki+1),m
      ipos1=(ki-1)*n
      ipos2=(kj-1)*n
      call Spearman(n,m,nmax,matmax,DMat,iordre,isymm,npermS,
     +   ipos1,ipos2,sx,sy,r,dflnk,idebug)
      rMantel(ki,kj)=r
      rMantel(kj,ki)=r
   20 continue
C
      do 30 ki=1,m
      rMantel(ki,ki)=1.0
   30 write(7,100) (rMantel(ki,kj), kj=1,m)
C Test par permutation des statistiques "rMantel"
      if(npermS.eq.0) goto 99
      write(*,*)
      if(langue.eq.1) then
         write(7,*)
         write(7,*) 'Tests permutationnels des statistiques de Mantel:'
         write(7,*) 'Probabilite unilaterale a droite (H1: r>0)'
         if(isymm.eq.1) then
            write(7,108) npermS
            else
            write(7,109) npermS
            endif
         else
         write(7,*)
         write(7,*) 'Permutation tests of the Mantel statistics:'
         write(7,*) 'One-tailed probability in the upper tail ',
     +'(H1: r>0)'
         if(isymm.eq.1) then
            write(7,208) npermS
            else
            write(7,209) npermS
            endif
         endif
      do 32 ki=1,(m-1)
      do 32 kj=(ki+1),m
   32 igetS(ki,kj)=1
C
C Debut des permutations - - - - - - - - - - - - - - -
      do 40 iperm=1,npermS
      call Permute(iseed,n,nmax,iordre)
      do 34 ki=1,(m-1)
      do 34 kj=(ki+1),m
      ipos1=(ki-1)*n
      ipos2=(kj-1)*n
      call Spearman(n,m,nmax,matmax,DMat,iordre,isymm,npermS,
     +   ipos1,ipos2,sx,sy,r,dflnk,idebug)
      if(r.ge.rMantel(ki,kj)) igetS(ki,kj)=igetS(ki,kj)+1
      if(idebug.eq.1) then
         write(*,*) 'r in Mantel   =',r,' rMantel =',rMantel(ki,kj)
         write(*,*) 'igetS =',igetS(ki,kj)
         endif
   34 continue
   40 continue
C Fin des permutations - - - - - - - - - - - - - - - -
C
C Calcul et impression des probabilites
      dflnpSp1=dfloat(npermS+1)
      do 42 ki=1,(m-1)
      do 42 kj=(ki+1),m
      probS(ki,kj)=dfloat(igetS(ki,kj))/dflnpSp1
   42 probS(kj,ki)=probS(ki,kj)
      do 44 ki=1,m
      probS(ki,ki)=1.0
   44 write(7,100) (probS(ki,kj), kj=1,m)
C     
   99 return
  100 format(30f10.5)
  110 format(30F8.0)
  101 format(/' Ressemblance entre les matrices de distance:'/
     +        ' Matrice de correlations de Mantel basees sur les ',
     +'rangs'/
     +        ' (correlations de Spearman)'/)
  201 format(/' Resemblance among the distance matrices:'/
     +        ' Matrix of Mantel correlations based upon ranks'/
     +        ' (Spearman correlations)'/)
  108 format(' Nombre de permutations =',i5,', matrices asymetriques'/)
  208 format(' Number of permutations =',i5,', non-symmetric ',
     +'matrices'/)
  109 format(' Nombre de permutations =',i5,', matrices symetriques'/)
  209 format(' Number of permutations =',i5,', symmetric matrices'/)
      end

      Subroutine Permute (iseed,n,nmax,iordre)
C This subroutine permutes the first 'n' values of vector 'iordre' at random
C in an equiprobable way. This property has been checked through intensive 
C simulations.
      Integer iseed,n,i,itemp,km1,m
      Integer iordre(nmax)
      m=n
      km1=n-1
      do 10 i=1,km1
    8    j = 1 + rand(iseed)*m
         if(j.gt.m) goto 8
         itemp = iordre(m)
         iordre(m) = iordre(j)
         iordre(j) = itemp
         m=m-1
   10 continue
      return
      end

      Subroutine Prelim(n,m,DMat,Table,nmax,matmax,k,kmax,T,liste,data,
     +       rang,isymm,iseed,idebug)
      Integer liste(kmax)
      Real*8 DMat(nmax*matmax,nmax),Table(matmax,kmax),T,data(kmax),
     +       rang(kmax)
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
C
C  Calcul du coefficient de concordance W --
C  Compute Kendall's W coefficient --
C
C  1. A tour de role, copier une ligne de 'Table' dans le vecteur 'data'.
C  2. Trier 'data' et le transformer en rangs.
C  3. Transcrire les distances transformees en rangs dans 'Table'.
C  4. Transcrire les distances transformees en rangs dans 'DMat'.
C
      T=0.0
      if(isymm.eq.1) then
         k=n*(n-1)
         else
         k=n*(n-1)/2         
         endif
C Etapes 1 a 4
      do 10 ii=1,m
      ipos=(ii-1)*n
      do 2 j=1,k
    2 data(j)=Table(ii,j)
      call Tri(k,kmax,liste,data,rang,T)
      do 4 j=1,k
    4 Table(ii,j)=data(j)
      jj=0
      if(isymm.eq.1) then
         do 6 i=1,(n-1)
         do 6 j=(i+1),n
         jj=jj+1
    6    DMat(ipos+i,j)=Table(ii,jj)
         do 7 i=2,n
         do 7 j=1,(i-1)
         jj=jj+1
    7    DMat(ipos+i,j)=Table(ii,jj)
        else
         do 8 i=1,(n-1)
         do 8 j=(i+1),n
         jj=jj+1
         DMat(ipos+i,j)=Table(ii,jj)
    8    DMat(ipos+j,i)=DMat(ipos+i,j)
        endif
      do 9 i=1,n
    9 DMat(ipos+i,i)=0.0         
   10 continue
C Initialisation du generateur de nombres aleatoires
      iseed=0
      do 20 i=1,100
   20 turn=rand(iseed)
      return
      end
 
      Subroutine ReadData(n,m,DMat,Table,w,nmax,matmax,kmax,isymm,
     +       langue)
      Real*8 DMat(nmax*matmax,nmax),Table(matmax,kmax),w(matmax)
      Real*8 wSum,dflm,dnorm
      Character*79 nom
C Lecture des donnees  
    2 if(langue.eq.1) then
         write(*,*)
     +      'Fichier(s) de donnees (matrices de distance carrees):'
         write(*,*)'(1) Un seul fichier contenant toutes les matrices?'
         write(*,*)'(2) Fichiers separes pour les differentes ',
     +'matrices?'
         else
         write(*,*)'Data file(s) (containing square distance ',
     +'matrices):'
         write(*,*)'(1) A single file containing all matrices?'
         write(*,*)'(2) Separate files for the different matrices?'
         endif
      read(*,*) ifich
      if((ifich.lt.1).or.(ifich.gt.2)) goto 2
      write(*,*)
C
    4 if(langue.eq.1) then
         write(*,*)
     +      'Toutes les matrices de distance sont-elles symetriques?'
         write(*,*)'(1) Au moins une matrice est asymetrique'
         write(*,*)'(2) Toutes les matrices sont symetriques'
         else
         write(*,*)'Are all distance matrices symmetric?'
         write(*,*)'(1) At least one matrix is non-symmetric'
         write(*,*)'(2) All distance matrices are symmetric'
         endif
      read(*,*) isymm
      if((isymm.lt.1).or.(isymm.gt.2)) goto 4
      write(*,*)
C
      if(langue.eq.1) then
         write(*,*)'Combien de matrices?'
         else
         write(*,*)'How many matrices?'
         endif
      read(*,*) m
      if(m.gt.matmax) then
         if(langue.eq.1) Stop 'Trop de matrices. Recompiler le prog.'
         if(langue.eq.2) Stop 'Too many matrices. Recompile program.'
         endif
C
      if(langue.eq.1) then
         write(*,*)'Combien d''objets?'
         else
         write(*,*)'How many objects?'
         endif
      read(*,*) n
      if(n.gt.nmax) then
         if(langue.eq.1) Stop 'Trop d''objets. Recompiler le prog.'
         if(langue.eq.2) Stop 'Too many objects. Recompile the prog.'
         endif
      goto (6,10) ifich
C Un seul fichier de donnees
    6 write(*,*)
      if(langue.eq.1) write(*,*) 'Nom du fichier de distances?'
      if(langue.eq.2) write(*,*) 'Name of the file of distances?'
      read(*,*) nom
      open(10,file=nom,status='OLD')
      inquire(unit=10,name=nom)
      if(langue.eq.1) then
         write(*,*) 'Fichier de distances: ', nom
         write(7,*) 'Fichier de distances: ', nom
         else
         write(*,*) 'Input distance file: ', nom
         write(7,*) 'Input distance file: ', nom
         endif
      do 8 i=1,n*m
    8 read(10,*) (DMat(i,j), j=1,n)
      close(10)
      goto 20
C Plusieurs fichiers de donnees
   10 do 16 ii=1,m
      ipos=(ii-1)*n
      write(*,*)
      if(langue.eq.1) write(*,102) ii
      if(langue.eq.2) write(*,202) ii
      read(*,*) nom
      open(10,file=nom,status='OLD')
      inquire(unit=10,name=nom)
      if(langue.eq.1) then
         write(*,*) 'Fichier de distances no',ii,': ',nom
         write(7,*) 'Fichier de distances no',ii,': ',nom
         else
         write(*,*) 'Input distance file No.',ii,': ',nom
         write(7,*) 'Input distance file No.',ii,': ',nom
         endif
      do 12 i=1,n
   12 read(10,*) (DMat(ipos+i,j), j=1,n)
      close(10)
   16 continue
C Transformer les matrices asymetriques si necessaire
   20 if(isymm.eq.1) then
         isymm=2
         do 22 ii=1,m   
         ipos=(ii-1)*n
         call TransMat(n,ii,DMat,nmax,matmax,ipos,isymm,langue)
   22    continue
         endif
C Transcrire les matrices de distance dans Table
      do 30 ii=1,m   
      ipos=(ii-1)*n
      jj=0
C     Matrices asymetriques
      if(isymm.eq.1) then
         do 24 i=1,(n-1)
         do 24 j=(i+1),n
         jj=jj+1
   24    Table(ii,jj)=DMat(ipos+i,j)
         do 26 i=2,n
         do 26 j=1,(i-1)
         jj=jj+1
   26    Table(ii,jj)=DMat(ipos+i,j)
C     Matrices symetriques
         else
         do 28 i=1,(n-1)
         do 28 j=(i+1),n
         jj=jj+1
   28    Table(ii,jj)=DMat(ipos+i,j)
         endif
   30 continue
C Ponderation des matrices de distance
   32 write(*,*)
      if(langue.eq.1) then
         write(*,*)'Ponderation des matrices de distance:'
         write(*,*)'(1) Toutes les matrices auront le meme poids'
         write(*,*)'(2) Ponderer les matrices de facon differente'
         else
         write(*,*)'Weighting the distance matrices:'
         write(*,*)'(1) Give the same weight to all matrices'
         write(*,*)'(2) Weight the matrices differently'
         endif
      read(*,*) iweight
      if((iweight.lt.1).or.(iweight.gt.2)) goto 32
      if(iweight.eq.2) then
         write(*,*)
         if(langue.eq.1) then
            write(*,*)'Tapez les poids sur une ou plusieurs lignes.'
            write(*,*)'Si la somme des poids n''est pas egale au ',
     +'nombre'
            write(*,*)'de matrices, les valeurs seront normalisees.'
            else
            write(*,*)'Type the list of weights on one or more lines.'
            write(*,*)'If the weights do not sum to the number of '
            write(*,*)'matrices, they will be normalized to that ',
     +'value.'
            endif
         read(*,*) (w(ii), ii=1,m)
         wSum=0.0
         do ii=1,m
            wSum=wSum+w(ii)
            enddo
         dflm=dfloat(m)
         if(wSum.eq.dflm) then
            if(langue.eq.1) then
               write(*,103) (w(ii), ii=1,m)
               write(7,103) (w(ii), ii=1,m)
               else
               write(*,203) (w(ii), ii=1,m)
               write(7,203) (w(ii), ii=1,m)
               endif
            else
            dnorm=dflm/wSum
            do ii=1,m
               w(ii)=w(ii)*dnorm
               enddo
            if(langue.eq.1) then
               write(*,104) (w(ii), ii=1,m)
               write(7,104) (w(ii), ii=1,m)
               else
               write(*,204) (w(ii), ii=1,m)
               write(7,204) (w(ii), ii=1,m)
               endif
            endif
         else
         if(langue.eq.1) then
            write(*,105)
            write(7,105)
            else
            write(*,205)
            write(7,205)
            endif         
         do ii=1,m
            w(ii)=1.0
            enddo
         endif
         write(7,*)
C
      return
C
  102 format(' Nom du fichier de distances no',i3)
  202 format(' Name of distance file No.',i3)
  103 format(/' Liste des poids:',50f8.5)
  203 format(/' List of weights:',50f8.5)
  104 format(/' Liste des poids normalises:',50f8.5)
  204 format(/' List of normalized weights:',50f8.5)
  105 format(/' Poids egaux pour toutes les matrices')
  205 format(/' Equal weights to all matrices')
      end

      Subroutine Spearman(n,m,nmax,matmax,DMat,iordre,isymm,npermS,
     +                    ipos1,ipos2,sx,sy,r,dflnk,idebug)
      Integer iordre(nmax)
      Real*8 DMat(nmax*matmax,nmax)
      Real*8 x,y,denom,sx,sy,sx2,sy2,sxy,r
      if(idebug.eq.1) write(*,102) (iordre(i), i=1,n)
      sx2=0.0
      sy2=0.0
      sxy=0.0
      if(isymm.eq.1) then
         do 14 i=1,(n-1)
         do 14 j=(i+1),n
         ia=iordre(i)
         ib=iordre(j)
         if(ia.lt.ib) then
            x=DMat(ipos1+ia,ib)
            else
            x=DMat(ipos1+ib,ia)
            endif
         y=DMat(ipos2+i,j)
         sx2=sx2+x*x
         sy2=sy2+y*y
         sxy=sxy+x*y
   14    continue
         do 16 i=2,n
         do 16 j=1,(i-1)
         ia=iordre(i)
         ib=iordre(j)
         if(ia.gt.ib) then
            x=DMat(ipos1+ia,ib)
            else
            x=DMat(ipos1+ib,ia)
            endif
         y=DMat(ipos2+i,j)
         sx2=sx2+x*x
         sy2=sy2+y*y
         sxy=sxy+x*y
   16    continue
        else
         do 18 i=1,(n-1)
         do 18 j=(i+1),n
         x=DMat(ipos1+iordre(i),iordre(j))
         y=DMat(ipos2+i,j)
         sx2=sx2+x*x
         sy2=sy2+y*y
         sxy=sxy+x*y
   18    continue
        endif
      denom=dsqrt((sx2-(sx*sx/dflnk))*(sy2-(sy*sy/dflnk)))
      r=(sxy-(sx*sy/dflnk))/denom
      if(idebug.eq.1) write(*,*) 'r in Spearman =',r
      return
  102 format(30i4)
      end

      Subroutine TransMat(n,ii,DMat,nmax,matmax,ipos,isymm,langue)
      Real*8 DMat(nmax*matmax,nmax),sum,temp
    2 if(langue.eq.1) then
         write(*,101) ii
         else
         write(*,201) ii
         endif
      read(*,*) ioption
      if((ioption.lt.0).or.(ioption.gt.4)) goto 2
      write(*,*)
      if(ioption.eq.0) goto 50
      goto (10,20,30,40) ioption
C Transformer en une matrice symetrique
   10 do 12 i=1,(n-1)
      do 12 j=(i+1),n
      temp=0.5*(DMat(ipos+i,j)+DMat(ipos+j,i))
      DMat(ipos+i,j)=temp
   12 DMat(ipos+j,i)=temp
      goto 50
C Transformer en un matrice antisymetrique ((sup-inf)/2)
   20 sum=0.0
      do 22 i=1,(n-1)
      do 22 j=(i+1),n
      temp=0.5*(DMat(ipos+i,j)-DMat(ipos+j,i))
      DMat(ipos+i,j)=temp
      DMat(ipos+j,i)=temp
   22 sum=sum+temp*temp
      if(sum.eq.0.0) then
         if(langue.eq.1) write(*,103)
         if(langue.eq.2) write(*,203)
         endif
      goto 50
C Transformer en un matrice antisymetrique ((inf-sup)/2)
   30 sum=0.0
      do 32 i=1,(n-1)
      do 32 j=(i+1),n
      temp=0.5*(DMat(ipos+j,i)-DMat(ipos+i,j))
      DMat(ipos+i,j)=temp
      DMat(ipos+j,i)=temp
   32 sum=sum+temp*temp
      if(sum.eq.0.0) then
         if(langue.eq.1) write(*,103)
         if(langue.eq.2) write(*,203)
         endif
      goto 50
C Une matrice demeure asymetrique
   40 isymm=1
      goto 50
C Imprimer la matrice?
   50 if(langue.eq.1) then
         write(*,102) ii,n,n
         else
         write(*,202) ii,n,n
         endif
      read(*,*) iprint
      if((iprint.lt.0).or.(iprint.gt.1)) goto 50
      write(*,*)
      if(iprint.eq.1) then
         do 58 i=1,n
   58    write(*,100) (DMat(ipos+i,j), j=1,n)
         endif
      return
  100 format(10f10.5)
  101 format(/' Matrice',i3,':'/
     + ' (0) Cette matrice est symetrique'/
     + ' (1) Transformer en une matrice symetrique en calculant la'/
     + '     moyenne des deux demi-matrices triangulaires ',
     +'(recommande)'/
     + ' (2) Transformer en une matrice anti-symetrique ',
     +'((sup-inf)/2)'/
     + ' (3) Transformer en une matrice anti-symetrique ',
     +'((inf-sup)/2)'/
     + ' (4) Conserver comme matrice asymetrique')
  201 format(/' Matrix',i3,':'/
     + ' (0) This matrix is symmetric'/
     + ' (1) Turn it into a symmetric matrix by averaging the'/
     + '     two triangular half-matrices (recommended solution)'/
     + ' (2) Turn it into a skew-symmetric matrix ((sup-inf)/2)'/
     + ' (3) Turn it into a skew-symmetric matrix ((inf-sup)/2)'/
     + ' (4) Leave it non-symmetric')
  102 format(' Matrice',i3,' (',i3,' lignes et ',i3,' colonnes):'/
     +       ' (0) Ne pas imprimer'/
     +       ' (1) Imprimer')
  202 format(' Matrix',i3,' (',i3,' rows and ',i3,' columns):'/
     +       ' (0) Do not print'/
     +       ' (1) Print')
  103 format(' Toutes les valeurs de cette matrice sont maintenant 0'/)
  203 format(' All values in this matrix are now 0'/)
      end
      
      Subroutine Tri(k,kmax,liste,data,rang,T)
      Integer liste(kmax)
      Real*8 data(kmax),rang(kmax),T,temp
C Tri: on remplace les distances par leur rang en tenant compte des ex-aequo.
C T = facteur de correction pour donnees ex aequo dans le calcul de W.
C rang(i) = rang de l'objet i; rang le plus eleve, lorsque donnees liees
C liste(icount) = nombre de valeurs de rang 'icount'
C k = nombre d'objets
      do 20 i=1,k
   20 liste(i)=0
C
      do 24 i=1,k
      icount=0
      do 22 j=1,k
      if(data(j).le.data(i)) icount=icount+1
   22 continue
      rang(i)=dfloat(icount)
      liste(icount)=liste(icount)+1
   24 continue
C Les 3 lignes suivantes sont utilisees pour la correction pour valeurs liees:
      do 26 i=1,k
      temp=dfloat(liste(i))
   26 T=T+temp*(temp+1.0)*(temp-1.0)
C On recalcule les rangs en tenant compte des donnees liees (valeur moyenne). 
C Puis, les rangs sont recopies dans le vecteur 'data'.
      do 30 i=1,k
      irang=rang(i)
      no=liste(irang)
      if(no.eq.1) then
         data(i)=rang(i)
         else
         itemp=0.0
         do 28 j=1,no
   28    itemp=itemp+irang-j+1
         data(i)=dfloat(itemp)/dfloat(no)
         endif
   30 continue
      return
      end

      Subroutine WKendall(m,k,Table,T,R,w,matmax,kmax,S,chisq,WKen,
     +                    iva,idebug)
      Real*8 Table(matmax,kmax),R(kmax),w(matmax)
      Real*8 denom,chisq,WKen,sumR,sumRsq,S,T,dflk
C
C Ce sous-programme calcule le khi-carre de Friedman ainsi que le W de Kendall.
C Le coefficient de concordance (W) de Kendall mesure la variance entre 
C les sommes de rangs des differentes colonnes de la matrice 'Table'.
C La variance est faible lorsqu'il y a accord entre les lignes; elle augmente 
C avec l'augmentation du desaccord entre elles, soit entre les matrices D.
C W est borne dans l'intervalle [0, 1]: 0 = desaccord complet, 1 = accord
C maximum. Khi-carre est la statistique (pivotale) a tester.
C On peut employer S, "chisq" ou W pour un test par permutations.
C
C La formule utilisant S'=sumRsq ne peut etre employee avec les poids.
C
      dflk=dfloat(k)
      dflm=dfloat(m)
      do 10 j=1,k
      R(j)=0.0
      do 10 i=1,m
   10 R(j)=R(j)+Table(i,j)*w(i)
C Calcul des statistiques khi-carre de Friedman et W de Kendall avec correction 
C pour donnees liees (Siegel 1956: 234; Siegel & Castellan 1988: 266).
      sumR  =0.0
      sumRsq=0.0
      do 12 j=1,k
      sumR=sumR+R(j)
   12 sumRsq=sumRsq+R(j)*R(j)
      S=sumRsq-(sumR*sumR/dflk)
      goto (20,30) iva
   20    denom=dflm*dflm*dflk*(dflk+1.0)*(dflk-1.0) - dflm*T
         if(idebug.eq.1) write(*,*) 'T =',T,' denom =',denom
         WKen=12.0*S/denom
         chisq=dflm*(dflk-1.0)*WKen
   30 return
      end

